@@ -615,12 +615,12 @@ class GOESNCBaseFileHandler(BaseFileHandler):
                                   mask_and_scale=False,
                                   chunks={"xc": CHUNK_SIZE, "yc": CHUNK_SIZE})
         self.sensor = "goes_imager"
-        self.nlines = self.nc.dims["yc"]
-        self.ncols = self.nc.dims["xc"]
+        self.nlines = self.nc.sizes["yc"]
+        self.ncols = self.nc.sizes["xc"]
         self.platform_name = self._get_platform_name(
             self.nc.attrs["Satellite Sensor"])
         self.platform_shortname = self.platform_name.replace("-", "").lower()
-        self.gvar_channel = int(self.nc["bands"].values)
+        self.gvar_channel = int(self.nc["bands"].item())
         self.sector = self._get_sector(channel=self.gvar_channel,
                                        nlines=self.nlines,
                                        ncols=self.ncols)
@@ -731,9 +731,9 @@ class GOESNCBaseFileHandler(BaseFileHandler):
     def start_time(self):
         """Start timestamp of the dataset."""
         dt = self.nc["time"].dt
-        return datetime(year=int(dt.year), month=int(dt.month), day=int(dt.day),
-                        hour=int(dt.hour), minute=int(dt.minute),
-                        second=int(dt.second), microsecond=int(dt.microsecond))
+        return datetime(year=int(dt.year.item()), month=int(dt.month.item()), day=int(dt.day.item()),
+                        hour=int(dt.hour.item()), minute=int(dt.minute.item()),
+                        second=int(dt.second.item()), microsecond=int(dt.microsecond.item()))
 
     @property
     def end_time(self):
@@ -1087,7 +1087,7 @@ class GOESEUMNCFileHandler(GOESNCBaseFileHandler):
 
         # Set proper dimension names
         data = data.rename({"xc": "x", "yc": "y"})
-        data = data.drop("time")
+        data = data.drop_vars("time")
 
         # Update metadata
         self._update_metadata(data, ds_info=info)
@@ -1124,8 +1124,8 @@ class GOESEUMGEONCFileHandler(BaseFileHandler):
                                   mask_and_scale=False,
                                   chunks={"xc": CHUNK_SIZE, "yc": CHUNK_SIZE})
         self.sensor = "goes_imager"
-        self.nlines = self.nc.dims["yc"]
-        self.ncols = self.nc.dims["xc"]
+        self.nlines = self.nc.sizes["yc"]
+        self.ncols = self.nc.sizes["xc"]
         self.platform_name = GOESNCBaseFileHandler._get_platform_name(
             self.nc.attrs["Satellite Sensor"])
         self.platform_shortname = self.platform_name.replace("-", "").lower()
