@@ -765,8 +765,8 @@ class HRITMSGFileHandler(HRITFileHandler):
         res.attrs["standard_name"] = info["standard_name"]
         res.attrs["platform_name"] = self.platform_name
         res.attrs["sensor"] = "seviri"
-        res.attrs["nominal_start_time"] = self.nominal_start_time,
-        res.attrs["nominal_end_time"] = self.nominal_end_time,
+        res.attrs["nominal_start_time"] = self.nominal_start_time
+        res.attrs["nominal_end_time"] = self.nominal_end_time
         res.attrs["time_parameters"] = {
             "nominal_start_time": self.nominal_start_time,
             "nominal_end_time": self.nominal_end_time,
