@@ -152,6 +152,7 @@ def _create_surf_refl_variables() -> dict[str, xr.DataArray]:
         "750m Surface Reflectance Band M1": xr.DataArray(m_data, dims=m_dims, attrs=sr_attrs),
     }
     for data_arr in data_arrs.values():
+        data_arr.encoding["chunksizes"] = data_arr.shape
         if "scale_factor" not in data_arr.attrs:
             continue
         data_arr.encoding["dtype"] = np.int16
@@ -219,6 +220,14 @@ def aod_file(tmp_path_factory: TempPathFactory) -> Path:
     data_vars = _create_continuous_variables(
         ("AOD550",)
     )
+    qc_data = np.zeros(data_vars["AOD550"].shape, dtype=np.int8)
+    qc_data[-1, -1] = 2
+    data_vars["QCAll"] = xr.DataArray(
+        qc_data,
+        dims=data_vars["AOD550"].dims,
+        attrs={"valid_range": [0, 3]},
+    )
+    data_vars["QCAll"].encoding["_FillValue"] = -128
     return _create_fake_file(tmp_path_factory, fn, data_vars)
 
 
@@ -370,7 +379,6 @@ class TestVIIRSJRRReader:
         ("var_names", "data_file"),
         [
             (("CldTopTemp", "CldTopHght", "CldTopPres"), lazy_fixture("cloud_height_file")),
-            (("AOD550",), lazy_fixture("aod_file")),
             (("VLST",), lazy_fixture("lst_file")),
         ]
     )
@@ -384,6 +392,31 @@ class TestVIIRSJRRReader:
         for var_name in var_names:
             _check_continuous_data_arr(scn[var_name])
 
+    @pytest.mark.parametrize(
+        ("aod_qc_filter", "exp_masked_pixel"),
+        [
+            (None, False),
+            (0, True),
+            (2, False)
+        ],
+    )
+    def test_get_aod_filtered(self, aod_file, aod_qc_filter, exp_masked_pixel):
+        """Test that the AOD product can be loaded and filtered."""
+        from satpy import Scene
+        bytes_in_m_row = 4 * 3200
+        with dask.config.set({"array.chunk-size": f"{bytes_in_m_row * 4}B"}):
+            scn = Scene(reader="viirs_edr", filenames=[aod_file], reader_kwargs={"aod_qc_filter": aod_qc_filter})
+            scn.load(["AOD550"])
+        _check_continuous_data_arr(scn["AOD550"])
+        data_np = scn["AOD550"].data.compute()
+        pixel_is_nan = np.isnan(data_np[-1, -1])
+        assert pixel_is_nan if exp_masked_pixel else not pixel_is_nan
+
+        # filtering should never affect geolocation
+        lons, lats = scn["AOD550"].attrs["area"].get_lonlats()
+        assert not np.isnan(lons[-1, -1].compute())
+        assert not np.isnan(lats[-1, -1].compute())
+
     @pytest.mark.parametrize(
         ("data_file", "exp_available"),
         [
